import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.media.*;
import javax.media.control.*;
import javax.media.format.*;
import javax.media.protocol.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * @author monius
 *
 */
public class JMFTerminal extends JPanel implements ActionListener
{

	/** audio video attributes */
	
	private CaptureDeviceInfo captureVideoDevice = null;
	private CaptureDeviceInfo captureAudioDevice = null;
	private DataSource videoDataSource = null;
	private DataSource audioDataSource = null;
	
	private static String defaultVideoFormatString = "size=320x240, encoding=yuv, maxdatalength=115200";
	private static String defaultAudioFormatString = "linear, 44100.0 hz, 16-bit, stereo, littleendian, signed";
	private static VideoFormat captureVideoFormat = null;
	private static AudioFormat captureAudioFormat = null;
	
	private DataSource cloneableVideoSource = null;
	private DataSource cloneableAudioSource = null;
	private DataSource clonedVideoSource = null;
	private DataSource clonedAudioSource = null;
	
	private DataSource mixedDataSource = null;
	private DataSource clonedMixedDataSource = null;

	private Player realPlayer = null;
	private Player recPlayer = null;
	private static Processor sinkProcessor = null;
	private static StateHelper sinkSh = null;
	private DataSink filewriter = null;
	private MediaLocator dest = null;
	
	private String lastRecordedFile = null;
	private String playDirectory = null;
	
	/** view */
	
	private JButton record;
	private JButton open;
	private JButton play;
	private JButton change;
	private ImageIcon openIcon;
	private ImageIcon stopIcon;
	private ImageIcon recIcon;
	private ImageIcon playIcon;
	private ImageIcon changeIcon;
	private JLabel state;
	private JPanel recFilePanel;
	private JLabel recFile;
	private JLabel recSpacer;
	private Border defaultBorder;
	
	private Image back;
	private JPanel playerPanel;
	private JPanel buts;
	private JPanel control;
	
	final static boolean debug = false;
	
	public JMFTerminal()
	{
		
		/** a littlebit dizine */
		
		setLayout(new BorderLayout());
    	setSize(340,390);
    	
    	openIcon = new ImageIcon(ClassLoader.getSystemResource("./pics/open.gif"));
    	playIcon = new ImageIcon(ClassLoader.getSystemResource("./pics/play.gif"));
    	stopIcon = new ImageIcon(ClassLoader.getSystemResource("./pics/stop.gif"));
    	recIcon = new ImageIcon(ClassLoader.getSystemResource("./pics/rec.gif"));
    	
    	record = new JButton(recIcon);
    	record.addActionListener(this);
    	defaultBorder = record.getBorder();
    	open = new JButton(openIcon);
    	open.addActionListener(this);
    	play = new JButton(playIcon);
    	play.addActionListener(this);
    	change = new JButton(new ImageIcon(ClassLoader.getSystemResource("./pics/recFile.gif")));
    	change.addActionListener(this);
    	recFile = new JLabel("testRec.mpg", JLabel.RIGHT);
    	recSpacer = new JLabel(" ");
    	state = new JLabel("   rendering cam & micro");
    	recFilePanel = new JPanel();
    	recFilePanel.setOpaque(false);
 		recFilePanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
 		recFilePanel.add(recFile);
    	recFilePanel.add(recSpacer);
    	recFilePanel.add(change);

		if(debug) System.out.println("border: "+record.getBorder().getClass().toString());
		if(debug) System.out.println("background: "+record.getBackground().getRed());
    	
    	back = getToolkit().getImage(ClassLoader.getSystemResource("./pics/back.jpg"));
    	
    	playerPanel = new JPanel();
    	playerPanel.setOpaque(false);
	   	playerPanel.setLayout(new FlowLayout());
    	
    	buts = new JPanel();
    	buts.setOpaque(false);
 		buts.setLayout(new FlowLayout(FlowLayout.LEFT));
    	buts.add(open);
    	buts.add(play);
    	buts.add(record);
    	buts.add(state);
    	
    	control = new JPanel();
    	control.setOpaque(false);
 		control.setLayout(new BoxLayout(control, BoxLayout.Y_AXIS));
 		control.add(buts);
    	control.add(recFilePanel);
    	
    	String originalDirectory = "" + ClassLoader.getSystemResource("./clips/");   	
    	playDirectory = originalDirectory.replaceAll("%20"," ");
    	
		/** detect video datasource */
		
		Vector videoDeviceList = CaptureDeviceManager.getDeviceList(new VideoFormat(VideoFormat.YUV));
		if(videoDeviceList.size() > 0)
			captureVideoDevice = (CaptureDeviceInfo)videoDeviceList.firstElement();
		else
		{
			if(debug) System.err.println("Hat net ganz geklappt mit WebCam finden");
			System.exit(-1);
		}
		
		Format deviceFormat[] = captureVideoDevice.getFormats();
		
		for (int i = 0; i < deviceFormat.length; i++)
		{	
			if (DeviceInfo.formatToString(deviceFormat[i]).indexOf(defaultVideoFormatString) >= 0)
			{
				captureVideoFormat = (VideoFormat) deviceFormat[i];
			}
		}
		
		try
		{
			videoDataSource = Manager.createDataSource(captureVideoDevice.getLocator());
		}
		catch (IOException e)
		{
			e.printStackTrace();
			System.exit(-1);
		}
		catch (NoDataSourceException e)
		{
			e.printStackTrace();
			System.exit(-1);
		}
		
		if (! DeviceInfo.setFormat(videoDataSource, captureVideoFormat))
		{
			if(debug) System.err.println("Error: unable to set video format - program aborted");
			System.exit(-1);
		}
		
		/** detect audio datasource */
		
		Vector AudioDeviceList = CaptureDeviceManager.getDeviceList(new AudioFormat(AudioFormat.LINEAR, 44100, 16, 2));
		if(AudioDeviceList.size() > 0)
			captureAudioDevice = (CaptureDeviceInfo)AudioDeviceList.firstElement();
		else
		{
			if(debug) System.err.println("Hat net ganz geklappt mit'm Micro finden");
			System.exit(-1);
		}
		
		Format deviceFormatA[] = captureAudioDevice.getFormats();
		
		for (int i = 0; i < deviceFormatA.length; i++)
		{	
			if (DeviceInfo.formatToString(deviceFormatA[i]).indexOf(defaultAudioFormatString) >= 0)
			{
				captureAudioFormat = (AudioFormat) deviceFormatA[i];
			}
		}
		
		try
		{
			audioDataSource = Manager.createDataSource(captureAudioDevice.getLocator());
		}
		catch (IOException e)
		{
			e.printStackTrace();
			System.exit(-1);
		}
		catch (NoDataSourceException e)
		{
			e.printStackTrace();
			System.exit(-1);
		}
		
		if (! DeviceInfo.setFormat(audioDataSource, captureAudioFormat))
		{
			if(debug) System.err.println("Error: unable to set audio format - program aborted");
			System.exit(-1);
		}
		
		/** create clones */
		
		cloneableVideoSource = Manager.createCloneableDataSource(videoDataSource);
		cloneableAudioSource = Manager.createCloneableDataSource(audioDataSource);
		
		clonedVideoSource = ((SourceCloneable)cloneableVideoSource).createClone();
		clonedAudioSource = ((SourceCloneable)cloneableAudioSource).createClone();
		
		/** merge both DataSources for rendering*/
		
		try
		{
			DataSource mergeArray[] = new DataSource[2];
			mergeArray[0] = cloneableVideoSource;
			mergeArray[1] = cloneableAudioSource;
			mixedDataSource = Manager.createMergingDataSource(mergeArray);
			
		}
		catch (IncompatibleSourceException e) 
		{
			e.printStackTrace();
			System.exit(-1);
		}
		
		
		/** create and start rendering player */
	
		try 
    	{
    		/** render current webcam & micro */
    		
			realPlayer = Manager.createRealizedPlayer(mixedDataSource);
	      	realPlayer.start();
	      	
			Component comp;
	      
	      	if ((comp = realPlayer.getVisualComponent()) != null)
	      	{
	        	playerPanel.removeAll();
	        	playerPanel.add(comp);
	        	playerPanel.doLayout();
	        	add(playerPanel, BorderLayout.NORTH);
	      	}
	      	if ((comp = realPlayer.getControlPanelComponent()) != null)
	      	{
	        	add(comp, BorderLayout.CENTER);
	      	}
	      	add(control, BorderLayout.SOUTH);
    	}
    	catch (Exception e)
    	{
    		e.printStackTrace();
    		System.exit(-1);
    	}
		
	}
	
	public void actionPerformed(ActionEvent e) 
  	{
    	JComponent c = (JComponent) e.getSource();
    
	    if(c == record) 
	    {
			if(open.getBackground().getRed() == 204)
			{
				if(record.getBackground().getRed() == 204)
				{
				    record.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED,Color.BLACK,Color.WHITE));
				    record.setBackground(Color.GRAY);
				    record.setIcon(stopIcon);
				    startRecording(recFile.getText());
				}
				else
				{
					record.setBorder(defaultBorder);
				    record.setBackground(new Color(204,204,204));
				    record.setIcon(recIcon);
				    stopRecording();
				}
			}
		}
		
		if(c == change) 
	    {
			if(open.getBackground().getRed() == 204 && record.getBackground().getRed() == 204)
			{
				saveFile();
			}
		}
	    
	    if(c == open) 
	    {
		    if(record.getBackground().getRed() == 204)
			{
			    if(open.getBackground().getRed() == 204)
				{
				    play.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED,Color.BLACK,Color.WHITE));
				    play.setBackground(Color.GRAY);
				    play.setIcon(stopIcon);
				    openFile();
				}
			}
	    }
	    if(c == play) 
	    {
		    if(record.getBackground().getRed() == 204)
			{
			    if(play.getBackground().getRed() == 204)
				{
				    play.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED,Color.BLACK,Color.WHITE));
				    play.setBackground(Color.GRAY);
				    play.setIcon(stopIcon);
				    playFile();
				}
				else
				{
					play.setBorder(defaultBorder);
				    play.setBackground(new Color(204,204,204));
				    play.setIcon(playIcon);
				    stopPlaying();
				}
			}
	    }
}

	/** choose file to be played */
	
	public void openFile()
	{
		FileDialog fileDialog = new FileDialog((Frame)this.getParent(), "play file:", FileDialog.LOAD);
		fileDialog.setLocation(320,220);
	
		fileDialog.show();
		
		String file = fileDialog.getFile();
		String loc = "file:/" + fileDialog.getDirectory();
		
		if(file != null) 
			startPlaying(loc, file);
		else
		{
			play.setBorder(defaultBorder);
			play.setBackground(new Color(204,204,204));
			play.setIcon(playIcon);
		}
	}
	
	public void playFile()
	{
		if(lastRecordedFile != null) 
			startPlaying(playDirectory, lastRecordedFile);
		else
		{
			play.setBorder(defaultBorder);
			play.setBackground(new Color(204,204,204));
			play.setIcon(playIcon);
			
			final JDialog alert = new JDialog((Frame)this.getParent(), "Alert !",true);
			alert.setLocation(320,220);
			alert.setSize(300,120);
			
			JLabel text = new JLabel("Es wurde noch nichts aufgenommen !");
			text.setLocation(10,10);
			text.setSize(250,20);
			JButton ok = new JButton("OK");
			ok.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent ae)
				{
					alert.dispose();
				}
			});
			ok.setLocation(10,40);
			ok.setSize(60,30);
			
			alert.getContentPane().setLayout(null);
			alert.getContentPane().add(text);
			alert.getContentPane().add(ok);
			alert.show();
		}
	}
	
	/** choose name and descriptor of file to be recorded to */
	
	public void saveFile()
	{
		final JDialog fileDialog = new JDialog((Frame)this.getParent(), true);
		fileDialog.setLocation(350,250);
		final JTextField name = new JTextField();
		String items[] = {".avi",".mpg"};
		final JComboBox avi = new JComboBox(items);
		avi.setSize(60,20);
		avi.setLocation(85,40);
		name.setSize(70,20);
		
		final JLabel text = new JLabel("change recFile to:");
		text.setSize(200,18);
		text.setLocation(10,10);
		name.setLocation(10,40);
		
		name.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(name.getText().length()>0)
				{
					fileDialog.dispose();
 					recFile.setText(name.getText()+avi.getSelectedItem());
 					
				}
				else
				{
					text.setText("enter name!!!");
				}
			}
		});
		
		fileDialog.getContentPane().setLayout(null);
		fileDialog.getContentPane().add(text);
		fileDialog.getContentPane().add(name);
		fileDialog.getContentPane().add(avi);
		
		fileDialog.setSize(160,110);
		
		fileDialog.addWindowListener(new WindowAdapter()
		{
		    public void windowClosing(WindowEvent e)
		    {
		    	record.setBorder(defaultBorder);
			    record.setBackground(new Color(204,204,204));
		    }
		});
		
		fileDialog.show();
	}

  
  	public void startRecording(String fileName)
  	{
  		state.setText("   recording");
  		
  		clonedVideoSource = ((SourceCloneable)cloneableVideoSource).createClone();
		clonedAudioSource = ((SourceCloneable)cloneableAudioSource).createClone();
  		
  		/** merge cloned video and data source */
  		
  		try
		{
			DataSource mergeArray[] = new DataSource[2];
			mergeArray[0] = clonedVideoSource;
			mergeArray[1] = clonedAudioSource;
			clonedMixedDataSource = Manager.createMergingDataSource(mergeArray);
			
		}
		catch (IncompatibleSourceException e) 
		{
			e.printStackTrace();
			System.exit(-1);
		}
  		
  		/** create processor */
  		
  		try
		{
			sinkProcessor = Manager.createProcessor(clonedMixedDataSource);
			sinkSh = new StateHelper(sinkProcessor);
		}
		catch(IOException e)
		{
			e.printStackTrace();
			System.exit(-1);
		}
		catch(NoProcessorException e)
		{
			e.printStackTrace();
			System.exit(-1);
		}
		
		if(!sinkSh.configure(10000))
		{
			System.exit(-1);
		}
			
		sinkProcessor.setContentDescriptor(new FileTypeDescriptor(FileTypeDescriptor.MSVIDEO));
			
		if(!sinkSh.realize(10000))
		{
			if(debug) System.err.println("Hat net ganz geklappt 5");
			System.exit(-1);
		}
		
		lastRecordedFile = fileName;
		
		DataSource source = sinkProcessor.getDataOutput();
		if(debug) System.out.println(ClassLoader.getSystemResource("./clips/")+fileName);
		String origin = ClassLoader.getSystemResource("./clips/")+fileName;
		String correct = origin.replaceAll("%20"," ");
		dest = new MediaLocator(correct);
//		dest = new MediaLocator("file://Dokumente und Einstellungen/monius/Eigene Dateien/java/jmf/jmf_terminal/clips/"+fileName);
		
		/** create datasink */
		
		try
		{
			filewriter = Manager.createDataSink(source, dest);
			filewriter.open();
		}
		catch(NoDataSinkException e)
		{
			e.printStackTrace();
			System.exit(-1);
		}
		catch(IOException e)
		{
			e.printStackTrace();
			System.exit(-1);
		}
		catch(SecurityException e)
		{
			e.printStackTrace();
			System.exit(-1);
		}
		
		if(!sinkSh.realize(10000))
		{
			if(debug) System.err.println("Hat net ganz geklappt 9");
			System.exit(-1);
		}
		
		StreamWriterControl swc = (StreamWriterControl)sinkProcessor.getControl("javax.media.control.StreamWriterControl");
		if(swc != null)
			swc.setStreamSizeLimit(50000000); // 50MB
		
		/** start datasink */
		
		try
		{
			filewriter.start();
			sinkSh.play();
			
			if(debug) System.out.println("Jetzt nimmt er auf!!!");
		}
		catch(IOException e)
		{
			e.printStackTrace();
			System.exit(-1);
		}

  	}
  	
  	public void stopRecording()
  	{
  		state.setText("   rendering cam & micro");
  		
  		 // sofortiges Aufhren der Aufnahme
		sinkSh.close();
		filewriter.close();
		if(debug) System.out.println("Jetzt hat er aufgehrt!!!");
  	}
  	
  	public void startPlaying(String location, String file)
  	{
  		state.setText("   playing "+ file);
  		
  		realPlayer.stop();
		
 		this.removeAll();
  		
  		try 
    	{
    		URL url = new URL(location+file);
    		DataSource clipSource = Manager.createDataSource(url);
    		recPlayer = Manager.createRealizedPlayer(clipSource);
    		
    		recPlayer.start();
	      	
			Component comp;
	      
	      	if ((comp = recPlayer.getVisualComponent()) != null)
	      	{
	        	playerPanel.removeAll();
	        	playerPanel.add(comp);
	        	playerPanel.doLayout();
	        	add(playerPanel, BorderLayout.NORTH);
	      	}
	      	if ((comp = recPlayer.getControlPanelComponent()) != null)
	      	{
	        	add(comp, BorderLayout.CENTER);
	      	}
	      	add(control, BorderLayout.SOUTH);
	      	
	      	this.doLayout();
	      	
    	}
    	catch (Exception e)
    	{
    		e.printStackTrace();
    		realPlayer.start();
 //   		System.exit(-1);
    	}
  	}
  	public void stopPlaying()
  	{
  		state.setText("   rendering cam & micro");
  		
  		recPlayer.stop();
  		recPlayer.close();
  		recPlayer.deallocate();
  		
  		removeAll();
  		
  		try 
    	{
	      	realPlayer.start();
	      	
			Component comp;
	      
	      	if ((comp = realPlayer.getVisualComponent()) != null)
	      	{
	        	playerPanel.removeAll();
	        	playerPanel.add(comp);
	        	add(playerPanel, BorderLayout.NORTH);
	      	}
	      	if ((comp = realPlayer.getControlPanelComponent()) != null)
	      	{
	        	add(comp, BorderLayout.CENTER);
	        	playerPanel.doLayout();
	      	}
	      	add(control, BorderLayout.SOUTH);
    	}
    	catch (Exception e)
    	{
    		e.printStackTrace();
    		System.exit(-1);
    	}
  	}
  	
  	/** only for background */
  	
  	public void paintComponent(Graphics g)
	{
		g.clearRect(0,0,this.getWidth(),this.getHeight());

		if (back != null)
		{
			g.drawImage(back,0,0,this);
			g.drawImage(back,320,0,this);
		}
		else
		{
			if(debug) System.out.println("Hintergrund ist null!");
		}
	}

	public static void main(String[] args)
	{
		Frame f = new Frame("JMF_Terminal");
		f.setResizable(false);
		JMFTerminal a = new JMFTerminal();
		
		
		f.addWindowListener(new WindowAdapter()
		{
		    public void windowClosing(WindowEvent e)
		    {
		    	System.exit(-1);
		    }
		});
    
	    f.add("Center",a);
	    f.pack();
	    f.setSize(new Dimension(340,388));
	    f.setLocation(300,200);
	    f.setVisible(true);
	}

}
